@@ -13,7 +13,7 @@ int main(int argc, char *argv[]) {
     });
 
     // Create entity, set value of position using reflection API
-    ecs_entity_t ent = ecs_new_entity(ecs, "ent");
+    ecs_entity_t ent = ecs_entity(ecs, { .name = "ent" });
     void *ptr = ecs_ensure_id(ecs, ent, Position);
 
     ecs_meta_cursor_t cur = ecs_meta_cursor(ecs, Position, ptr);
