@@ -368,7 +368,7 @@ const char* flecs_json_skip_object(
         ecs_assert(json != NULL, ECS_INTERNAL_ERROR, NULL);
     }
 
-    ecs_parser_error(desc->name, desc->expr, 0, 
+    ecs_parser_error(desc->name, json, 0, 
         "expected }, got end of string");
     return NULL;
 }
@@ -558,8 +558,10 @@ void flecs_json_label(
     if (lbl) {
         flecs_json_string_escape(buf, lbl);
     } else {
+        ecs_strbuf_appendch(buf, '"');
         ecs_strbuf_appendch(buf, '#');
         ecs_strbuf_appendint(buf, (uint32_t)e);
+        ecs_strbuf_appendch(buf, '"');
     }
 }
 
