@@ -467,31 +467,6 @@ uint64_t flecs_query_cache_group_by_cascade(
     return flecs_ito(uint64_t, depth);
 }
 
-static
-void flecs_query_cache_add_ref(
-    ecs_world_t *world,
-    ecs_query_impl_t *impl,
-    ecs_query_cache_table_match_t *qm,
-    ecs_entity_t component,
-    ecs_entity_t entity,
-    ecs_size_t size)
-{
-    ecs_assert(entity != 0, ECS_INTERNAL_ERROR, NULL);
-    ecs_ref_t *ref = ecs_vec_append_t(&world->allocator, &qm->refs, ecs_ref_t);
-    ecs_assert(entity != 0, ECS_INTERNAL_ERROR, NULL);
-    
-    if (size) {
-        *ref = ecs_ref_init_id(world, entity, component);
-    } else {
-        *ref = (ecs_ref_t){
-            .entity = entity,
-            .id = 0
-        };
-    }
-
-    impl->pub.flags |= EcsQueryHasRefs;
-}
-
 static
 ecs_query_cache_table_match_t* flecs_query_cache_add_table_match(
     ecs_query_cache_t *cache,
@@ -502,12 +477,9 @@ ecs_query_cache_table_match_t* flecs_query_cache_add_table_match(
      * the query contains wildcards. */
     ecs_query_cache_table_match_t *qm = flecs_query_cache_cache_add(
         cache->query->world, qt);
+    
     qm->table = table;
-
-    qm->columns = flecs_balloc(&cache->allocators.columns);
-    qm->storage_columns = flecs_balloc(&cache->allocators.columns);
-    qm->ids = flecs_balloc(&cache->allocators.ids);
-    qm->sources = flecs_balloc(&cache->allocators.sources);
+    qm->trs = flecs_balloc(&cache->allocators.trs);
 
     /* Insert match to iteration list if table is not empty */
     if (!table || ecs_table_count(table) != 0 || 
@@ -521,70 +493,60 @@ ecs_query_cache_table_match_t* flecs_query_cache_add_table_match(
 
 static
 void flecs_query_cache_set_table_match(
-    ecs_world_t *world,
-    ecs_query_impl_t *impl,
     ecs_query_cache_t *cache,
     ecs_query_cache_table_match_t *qm,
-    ecs_table_t *table,
     ecs_iter_t *it)
 {
-    ecs_allocator_t *a = &world->allocator;
     ecs_query_t *query = cache->query;
-    int32_t i, term_count = query->term_count;
-    int32_t field_count = query->field_count;
-    ecs_term_t *terms = query->terms;
+    int8_t i, field_count = query->field_count;
+
+    ecs_assert(field_count > 0, ECS_INTERNAL_ERROR, NULL);
 
     /* Reset resources in case this is an existing record */
-    ecs_vec_reset_t(a, &qm->refs, ecs_ref_t);
-    ecs_os_memcpy_n(qm->columns, it->columns, int32_t, field_count);
-    ecs_os_memcpy_n(qm->ids, it->ids, ecs_id_t, field_count);
-    ecs_os_memcpy_n(qm->sources, it->sources, ecs_entity_t, field_count);
+    ecs_os_memcpy_n(ECS_CONST_CAST(ecs_table_record_t**, qm->trs), 
+        it->trs, ecs_table_record_t*, field_count);
+
+    /* Find out whether to store result-specific ids array or fixed array */
+    ecs_id_t *ids = cache->query->ids;
+    for (i = 0; i < field_count; i ++) {
+        if (it->ids[i] != ids[i]) {
+            break;
+        }
+    }
+
+    if (i != field_count) {
+        if (qm->ids == ids || !qm->ids) {
+            qm->ids = flecs_balloc(&cache->allocators.ids);
+        }
+        ecs_os_memcpy_n(qm->ids, it->ids, ecs_id_t, field_count);
+    } else {
+        if (qm->ids != ids) {
+            flecs_bfree(&cache->allocators.ids, qm->ids);
+            qm->ids = ids;
+        }
+    }
+
+    /* Find out whether to store result-specific sources array or fixed array */
+    for (i = 0; i < field_count; i ++) {
+        if (it->sources[i]) {
+            break;
+        }
+    }
+
+    if (i != field_count) {
+        if (qm->sources == cache->sources || !qm->sources) {
+            qm->sources = flecs_balloc(&cache->allocators.sources);
+        }
+        ecs_os_memcpy_n(qm->sources, it->sources, ecs_entity_t, field_count);
+    } else {
+        if (qm->sources != cache->sources) {
+            flecs_bfree(&cache->allocators.sources, qm->sources);
+            qm->sources = cache->sources;
+        }
+    }
+
     qm->set_fields = it->set_fields;
     qm->up_fields = it->up_fields;
-
-    if (table) {
-        /* Initialize storage columns for faster access to component storage */
-        for (i = 0; i < field_count; i ++) {
-            int32_t column = it->columns[i];
-            if (!ecs_field_is_set(it, i) || terms[i].inout == EcsInOutNone) {
-                qm->storage_columns[i] = -1;
-                continue;
-            }
-
-            ecs_entity_t src = it->sources[i];
-            if (!src) {
-                qm->storage_columns[i] = ecs_table_type_to_column_index(
-                    table, column);
-            } else {
-                /* Shared field (not from table) */
-                qm->storage_columns[i] = -2;
-            }
-        }
-    }
-
-    /* Add references for substituted terms */
-    for (i = 0; i < term_count; i ++) {
-        int32_t field = terms[i].field_index;
-        ecs_entity_t src = it->sources[field];
-        ecs_size_t size = 0;
-        if (it->sizes) {
-            size = it->sizes[field];
-        }
-        if (src) {
-            ecs_id_t id = it->ids[field];
-            ecs_assert(ecs_is_valid(world, src), ECS_INTERNAL_ERROR, NULL);
-
-            if (id) {
-                flecs_query_cache_add_ref(
-                    world, impl, qm, id, src, size);
-
-                /* Use column index to bind term and ref */
-                if (qm->set_fields & (1llu << field)) {
-                    qm->columns[field] = ecs_vec_count(&qm->refs) - 1;
-                }
-            }
-        }
-    }
 }
 
 static
@@ -613,14 +575,12 @@ ecs_query_cache_table_t* flecs_query_cache_table_insert(
 static
 void flecs_query_cache_match_tables(
     ecs_world_t *world,
-    ecs_query_impl_t *impl,
     ecs_query_cache_t *cache)
 {
     ecs_table_t *table = NULL;
     ecs_query_cache_table_t *qt = NULL;
 
     ecs_iter_t it = ecs_query_iter(world, cache->query);
-    ECS_BIT_SET(it.flags, EcsIterIsInstanced);
     ECS_BIT_SET(it.flags, EcsIterNoData);
     ECS_BIT_SET(it.flags, EcsIterTableOnly);
 
@@ -633,14 +593,13 @@ void flecs_query_cache_match_tables(
 
         ecs_query_cache_table_match_t *qm = 
             flecs_query_cache_add_table_match(cache, qt, table);
-        flecs_query_cache_set_table_match(world, impl, cache, qm, table, &it);
+        flecs_query_cache_set_table_match(cache, qm, &it);
     }
 }
 
 static
 bool flecs_query_cache_match_table(
     ecs_world_t *world,
-    ecs_query_impl_t *impl,
     ecs_query_cache_t *cache,
     ecs_table_t *table)
 {
@@ -654,7 +613,6 @@ bool flecs_query_cache_match_table(
     /* Iterate uncached query for table to check if it matches. If this is a
      * wildcard query, a table can match multiple times. */
     ecs_iter_t it = flecs_query_iter(world, q);
-    it.flags |= EcsIterIsInstanced;
     it.flags |= EcsIterNoData;
     ecs_iter_set_var_as_table(&it, 0, table);
 
@@ -667,7 +625,7 @@ bool flecs_query_cache_match_table(
 
         ecs_query_cache_table_match_t *qm = flecs_query_cache_add_table_match(
             cache, qt, table);
-        flecs_query_cache_set_table_match(world, impl, cache, qm, table, &it);
+        flecs_query_cache_set_table_match(cache, qm, &it);
     }
 
     return qt != NULL;
@@ -832,19 +790,23 @@ void flecs_query_cache_table_match_free(
     ecs_world_t *world = cache->query->world;
 
     for (cur = first; cur != NULL; cur = next) {
-        flecs_bfree(&cache->allocators.columns, cur->columns);
-        flecs_bfree(&cache->allocators.columns, cur->storage_columns);
-        flecs_bfree(&cache->allocators.ids, cur->ids);
-        flecs_bfree(&cache->allocators.sources, cur->sources);
+        flecs_bfree(&cache->allocators.trs, ECS_CONST_CAST(void*, cur->trs));
+
+        if (cur->ids != cache->query->ids) {
+            flecs_bfree(&cache->allocators.ids, cur->ids);
+        }
+
+        if (cur->sources != cache->sources) {
+            flecs_bfree(&cache->allocators.sources, cur->sources);
+        }
 
         if (cur->monitor) {
             flecs_bfree(&cache->allocators.monitors, cur->monitor);
         }
+
         if (!elem->hdr.empty) {
             flecs_query_cache_remove_table_node(cache, cur);
         }
-        
-        ecs_vec_fini_t(&world->allocator, &cur->refs, ecs_ref_t);
 
         next = cur->next_match;
 
@@ -895,7 +857,6 @@ void flecs_query_cache_rematch_tables(
     cache->monitor_generation = world->monitor_generation;
 
     it = ecs_query_iter(world, cache->query);
-    ECS_BIT_SET(it.flags, EcsIterIsInstanced);
     ECS_BIT_SET(it.flags, EcsIterNoData);
 
     world->info.rematch_count_total ++;
@@ -933,7 +894,7 @@ void flecs_query_cache_rematch_tables(
             qm = flecs_query_cache_add_table_match(cache, qt, table);
         }
 
-        flecs_query_cache_set_table_match(world, impl, cache, qm, table, &it);
+        flecs_query_cache_set_table_match(cache, qm, &it);
 
         if (table && ecs_table_count(table) && cache->group_by_callback) {
             if (flecs_query_cache_get_group_id(cache, table) != qm->group_id) {
@@ -978,7 +939,7 @@ void flecs_query_cache_notify(
     switch(event->kind) {
     case EcsQueryTableMatch:
         /* Creation of new table */
-        flecs_query_cache_match_table(world, impl, cache, event->table);
+        flecs_query_cache_match_table(world, cache, event->table);
         break;
     case EcsQueryTableUnmatch:
         /* Deletion of table */
@@ -1096,7 +1057,7 @@ void flecs_query_cache_on_event(
 
     if (event == EcsOnTableCreate) {
         /* Creation of new table */
-        if (flecs_query_cache_match_table(world, impl, cache, table)) {
+        if (flecs_query_cache_match_table(world, cache, table)) {
             if (ecs_should_log_3()) {
                 char *table_str = ecs_table_str(world, table);
                 ecs_dbg_3("query cache event: %s for [%s]", 
@@ -1158,8 +1119,8 @@ void flecs_query_cache_allocators_init(
 {
     int32_t field_count = cache->query->field_count;
     if (field_count) {
-        flecs_ballocator_init(&cache->allocators.columns, 
-            field_count * ECS_SIZEOF(int32_t));
+        flecs_ballocator_init(&cache->allocators.trs, 
+            field_count * ECS_SIZEOF(ecs_table_record_t*));
         flecs_ballocator_init(&cache->allocators.ids, 
             field_count * ECS_SIZEOF(ecs_id_t));
         flecs_ballocator_init(&cache->allocators.sources,
@@ -1175,7 +1136,7 @@ void flecs_query_cache_allocators_fini(
 {
     int32_t field_count = cache->query->field_count;
     if (field_count) {
-        flecs_ballocator_fini(&cache->allocators.columns);
+        flecs_ballocator_fini(&cache->allocators.trs);
         flecs_ballocator_fini(&cache->allocators.ids);
         flecs_ballocator_fini(&cache->allocators.sources);
         flecs_ballocator_fini(&cache->allocators.monitors);
@@ -1220,6 +1181,11 @@ void flecs_query_cache_fini(
     ecs_map_fini(&cache->groups);
 
     ecs_vec_fini_t(NULL, &cache->table_slices, ecs_query_cache_table_match_t);
+    
+    if (cache->query->term_count) {
+        flecs_bfree(&cache->allocators.sources, cache->sources);
+    }
+
     flecs_query_cache_allocators_fini(cache);
     ecs_query_fini(cache->query);
 
@@ -1280,6 +1246,13 @@ ecs_query_cache_t* flecs_query_cache_init(
 
     flecs_query_cache_allocators_init(result);
 
+    /* Zero'd out sources array that's used for results that only match $this. 
+     * This reduces the amount of memory used by the cache, and improves CPU
+     * cache locality during iteration when doing source checks. */
+    if (result->query->term_count) {
+        result->sources = flecs_bcalloc(&result->allocators.sources);
+    }
+
     if (q->term_count) {
         observer_desc.run = flecs_query_cache_on_event;
         observer_desc.ctx = impl;
@@ -1292,7 +1265,6 @@ ecs_query_cache_t* flecs_query_cache_init(
 
         observer_desc.events[event_index ++] = EcsOnTableCreate;
         observer_desc.events[event_index ++] = EcsOnTableDelete;
-        observer_desc.query.flags |= EcsQueryNoData|EcsQueryIsInstanced;
         observer_desc.flags_ = EcsObserverBypassQuery;
 
         /* ecs_query_init could have moved away resources from the terms array
@@ -1347,7 +1319,7 @@ ecs_query_cache_t* flecs_query_cache_init(
      * change back to being in sync before processing pending events. */
     ecs_run_aperiodic(world, EcsAperiodicEmptyTables);
     ecs_table_cache_init(world, &result->cache);
-    flecs_query_cache_match_tables(world, impl, result);
+    flecs_query_cache_match_tables(world, result);
 
     if (const_desc->order_by_callback) {
         if (flecs_query_cache_order_by(world, impl, 
@@ -1384,7 +1356,8 @@ void ecs_iter_set_group(
 {
     ecs_check(it != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_check(it->next == ecs_query_next, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_PARAMETER, NULL);
+    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_PARAMETER, 
+        "cannot set group during iteration");
 
     ecs_query_iter_t *qit = &it->priv_.iter.query;
     ecs_query_impl_t *q = flecs_query_impl(qit->query);
