@@ -195,7 +195,7 @@ repeat: {}
     }
 
     flecs_query_var_set_range(op, op->src.var, table, 0, 0, ctx);
-    flecs_query_it_set_column(it, field, tr->index);
+    flecs_query_it_set_tr(it, field, tr);
     return true;
 }
 
@@ -233,7 +233,7 @@ bool flecs_query_with_id(
         return false;
     }
 
-    flecs_query_it_set_column(it, field, tr->index);
+    flecs_query_it_set_tr(it, field, tr);
     return true;
 }
 
@@ -273,7 +273,7 @@ bool flecs_query_up_select(
             ecs_pair(op_ctx->trav, EcsWildcard));
     }
 
-    if (!op_ctx->idr_trav || !flecs_table_cache_count(&op_ctx->idr_trav->cache)){
+    if (!op_ctx->idr_trav || !flecs_table_cache_all_count(&op_ctx->idr_trav->cache)){
         if (!self) {
             return false;
         } else if (kind == FlecsQueryUpSelectId) {
@@ -362,7 +362,7 @@ bool flecs_query_up_select(
             } else {
                 int32_t row;
                 ecs_entity_t entity = 0;
-                ecs_entity_t *entities = flecs_table_entities_array(table);
+                const ecs_entity_t *entities = ecs_table_entities(table);
 
                 for (row = op_ctx->row; row < op_ctx->end; row ++) {
                     entity = entities[row];
@@ -457,7 +457,7 @@ bool flecs_query_up_with(
 
         it->sources[op->field_index] = flecs_entities_get_alive(
             ctx->world, up->src);
-        it->columns[op->field_index] = up->column;
+        it->trs[op->field_index] = up->tr;
         it->ids[op->field_index] = up->id;
         flecs_query_set_vars(op, up->id, ctx);
         flecs_set_source_set_flag(it, op->field_index);
@@ -640,51 +640,9 @@ bool flecs_query_triv(
     ctx->written[ctx->op_index + 1] |= 1ull;
     if (written & 1ull) {
         flecs_query_set_iter_this(ctx->it, ctx);
-        return flecs_query_trivial_test(
-            ctx->query, ctx, !redo, termset);
+        return flecs_query_trivial_test(ctx, redo, termset);
     } else {
-        return flecs_query_trivial_search_nodata(
-            ctx->query, ctx, op_ctx, !redo, termset);
-    }
-}
-
-static
-bool flecs_query_triv_data(
-    const ecs_query_op_t *op,
-    bool redo,
-    const ecs_query_run_ctx_t *ctx)
-{
-    ecs_query_trivial_ctx_t *op_ctx = flecs_op_ctx(ctx, trivial);
-    ecs_flags64_t termset = op->src.entity;
-    uint64_t written = ctx->written[ctx->op_index];
-    ctx->written[ctx->op_index + 1] |= 1ull;
-    if (written & 1ull) {
-        flecs_query_set_iter_this(ctx->it, ctx);
-        return flecs_query_trivial_test(
-            ctx->query, ctx, !redo, termset);
-    } else {
-        return flecs_query_trivial_search(
-            ctx->query, ctx, op_ctx, !redo, termset);
-    }
-}
-
-static
-bool flecs_query_triv_wildcard(
-    const ecs_query_op_t *op,
-    bool redo,
-    const ecs_query_run_ctx_t *ctx)
-{
-    ecs_query_trivial_ctx_t *op_ctx = flecs_op_ctx(ctx, trivial);
-    ecs_flags64_t termset = op->src.entity;
-    uint64_t written = ctx->written[ctx->op_index];
-    ctx->written[ctx->op_index + 1] |= 1ull;
-    if (written & 1ull) {
-        flecs_query_set_iter_this(ctx->it, ctx);
-        return flecs_query_trivial_test_w_wildcards(
-            ctx->query, ctx, !redo, termset);
-    } else {
-        return flecs_query_trivial_search_w_wildcards(
-            ctx->query, ctx, op_ctx, !redo, termset);
+        return flecs_query_trivial_search(ctx, op_ctx, redo, termset);
     }
 }
 
@@ -700,27 +658,9 @@ bool flecs_query_cache(
     uint64_t written = ctx->written[ctx->op_index];
     ctx->written[ctx->op_index + 1] |= 1ull;
     if (written & 1ull) {
-        return flecs_query_cache_test(ctx->query, ctx, !redo);
+        return flecs_query_cache_test(ctx, redo);
     } else {
-        return flecs_query_cache_search(ctx->query, ctx);
-    }
-}
-
-static
-bool flecs_query_cache_data(
-    const ecs_query_op_t *op,
-    bool redo,
-    const ecs_query_run_ctx_t *ctx)
-{
-    (void)op;
-    (void)redo;
-
-    uint64_t written = ctx->written[ctx->op_index];
-    ctx->written[ctx->op_index + 1] |= 1ull;
-    if (written & 1ull) {
-        return flecs_query_cache_data_test(ctx->query, ctx, !redo);
-    } else {
-        return flecs_query_cache_data_search(ctx->query, ctx);
+        return flecs_query_cache_search(ctx);
     }
 }
 
@@ -735,26 +675,9 @@ bool flecs_query_is_cache(
     uint64_t written = ctx->written[ctx->op_index];
     ctx->written[ctx->op_index + 1] |= 1ull;
     if (written & 1ull) {
-        return flecs_query_is_cache_test(ctx->query, ctx, !redo);
+        return flecs_query_is_cache_test(ctx, redo);
     } else {
-        return flecs_query_is_cache_search(ctx->query, ctx);
-    }
-}
-
-static
-bool flecs_query_is_cache_data(
-    const ecs_query_op_t *op,
-    bool redo,
-    const ecs_query_run_ctx_t *ctx)
-{
-    (void)op;
-
-    uint64_t written = ctx->written[ctx->op_index];
-    ctx->written[ctx->op_index + 1] |= 1ull;
-    if (written & 1ull) {
-        return flecs_query_is_cache_data_test(ctx->query, ctx, !redo);
-    } else {
-        return flecs_query_is_cache_data_search(ctx->query, ctx);
+        return flecs_query_is_cache_search(ctx);
     }
 }
 
@@ -973,7 +896,7 @@ bool flecs_query_ids(
         ecs_iter_t *it = ctx->it;
         it->ids[op->field_index] = id;
         it->sources[op->field_index] = EcsWildcard;
-        it->columns[op->field_index] = -1; /* Mark field as set */
+        it->trs[op->field_index] = NULL; /* Mark field as set */
     }
 
     return true;
@@ -1101,7 +1024,7 @@ bool flecs_query_each(
         if (end) {
             end += range.offset;
         } else {
-            end = table->data.entities.count;
+            end = ecs_table_count(table);
         }
         row = ++ op_ctx->row;
         if (op_ctx->row >= end) {
@@ -1110,7 +1033,7 @@ bool flecs_query_each(
     }
 
     ecs_assert(row < ecs_table_count(table), ECS_INTERNAL_ERROR, NULL);
-    ecs_entity_t *entities = table->data.entities.array;
+    const ecs_entity_t *entities = ecs_table_entities(table);
     flecs_query_var_set_entity(op, op->src.var, entities[row], ctx);
 
     return true;
@@ -1372,6 +1295,8 @@ void flecs_query_reset_after_block(
         it->ids[field] = id;
     }
 
+    it->trs[field] = NULL;
+
     /* Reset variables */
     if (flags_1st & EcsQueryIsVar) {
         if (!flecs_ref_is_written(op, &op->first, EcsQueryFirst, written_cur)){
@@ -1513,7 +1438,7 @@ bool flecs_query_select_or(
             ecs_var_t old_table_var;
             ecs_os_memset_t(&old_table_var, 0, ecs_var_t);
             bool restore_table_var = false;
-            
+
             if (prev_op->flags & (EcsQueryIsVar << EcsQuerySrc)) {
                 if (first_op->src.var != prev_op->src.var) {
                     restore_table_var = true;
@@ -1523,6 +1448,10 @@ bool flecs_query_select_or(
                 }
             }
 
+            int16_t field_index = op->field_index;
+            ecs_id_t prev_id = it->ids[field_index];
+            const ecs_table_record_t *prev_tr = it->trs[field_index];
+
             do {
                 ctx->written[prev] = ctx->written[last];
 
@@ -1548,6 +1477,9 @@ bool flecs_query_select_or(
                 continue;
             }
 
+            /* Restore id in case op set it */
+            it->ids[field_index] = prev_id;
+            it->trs[field_index] = prev_tr;
             break;
         }
 
@@ -1678,7 +1610,7 @@ bool flecs_query_if_set(
     ecs_query_run_ctx_t *ctx)
 {
     ecs_iter_t *it = ctx->it;
-    uint8_t field_index = flecs_ito(uint8_t, op->other);
+    int8_t field_index = flecs_ito(int8_t, op->other);
 
     ecs_query_ctrl_ctx_t *op_ctx = flecs_op_ctx(ctx, ctrl);
     if (!redo) {
@@ -1713,12 +1645,8 @@ bool flecs_query_dispatch(
     case EcsQueryAndId: return flecs_query_and_id(op, redo, ctx);
     case EcsQueryAndAny: return flecs_query_and_any(op, redo, ctx);
     case EcsQueryTriv: return flecs_query_triv(op, redo, ctx);
-    case EcsQueryTrivData: return flecs_query_triv_data(op, redo, ctx);
-    case EcsQueryTrivWildcard: return flecs_query_triv_wildcard(op, redo, ctx);
     case EcsQueryCache: return flecs_query_cache(op, redo, ctx);
     case EcsQueryIsCache: return flecs_query_is_cache(op, redo, ctx);
-    case EcsQueryCacheData: return flecs_query_cache_data(op, redo, ctx);
-    case EcsQueryIsCacheData: return flecs_query_is_cache_data(op, redo, ctx);
     case EcsQueryOnlyAny: return flecs_query_only_any(op, redo, ctx);
     case EcsQueryUp: return flecs_query_up(op, redo, ctx);
     case EcsQueryUpId: return flecs_query_up_id(op, redo, ctx);
@@ -1764,9 +1692,6 @@ bool flecs_query_dispatch(
     case EcsQuerySetId: return flecs_query_setid(op, redo, ctx);
     case EcsQueryContain: return flecs_query_contain(op, redo, ctx);
     case EcsQueryPairEq: return flecs_query_pair_eq(op, redo, ctx);
-    case EcsQueryPopulate: return flecs_query_populate(op, redo, ctx);
-    case EcsQueryPopulateSelf: return flecs_query_populate_self(op, redo, ctx);
-    case EcsQueryPopulateSparse: return flecs_query_populate_sparse(op, redo, ctx);
     case EcsQueryYield: return false;
     case EcsQueryNothing: return false;
     }
