@@ -6,20 +6,37 @@
 #include "../../private_api.h"
 
 static
-void flecs_query_iter_init(
+void flecs_query_iter_run_ctx_init(
+    ecs_iter_t *it,
     ecs_query_run_ctx_t *ctx)
 {
-    ecs_assert(ctx->written != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_iter_t *it = ctx->it;
+    ecs_query_iter_t *qit = &it->priv_.iter.query;
+    ecs_query_impl_t *impl = ECS_CONST_CAST(ecs_query_impl_t*, qit->query);
+    ctx->world = it->real_world;
+    ctx->query = impl;
+    ctx->it = it;
+    ctx->vars = qit->vars;
+    ctx->query_vars = qit->query_vars;
+    ctx->written = qit->written;
+    ctx->op_ctx = qit->op_ctx;
+    ctx->qit = qit;
+}
 
-    const ecs_query_impl_t *query = ctx->query;
+void flecs_query_iter_constrain(
+    ecs_iter_t *it)
+{
+    ecs_query_run_ctx_t ctx;
+    flecs_query_iter_run_ctx_init(it, &ctx);
+    ecs_assert(ctx.written != NULL, ECS_INTERNAL_ERROR, NULL);
+
+    const ecs_query_impl_t *query = ctx.query;
     const ecs_query_t *q = &query->pub;
     ecs_flags64_t it_written = it->constrained_vars;
-    ctx->written[0] = it_written;
-    if (it_written && ctx->query->src_vars) {
+    ctx.written[0] = it_written;
+    if (it_written && ctx.query->src_vars) {
         /* If variables were constrained, check if there are any table
          * variables that have a constrained entity variable. */
-        ecs_var_t *vars = ctx->vars;
+        ecs_var_t *vars = ctx.vars;
         int32_t i, count = q->field_count;
         for (i = 0; i < count; i ++) {
             ecs_var_id_t var_id = query->src_vars[i];
@@ -33,48 +50,58 @@ void flecs_query_iter_init(
 
             /* Initialize table variable with constrained entity variable */
             ecs_var_t *tvar = &vars[var->table_id];
-            tvar->range = flecs_range_from_entity(vars[var_id].entity, ctx);
-            ctx->written[0] |= (1ull << var->table_id); /* Mark as written */
+            tvar->range = flecs_range_from_entity(vars[var_id].entity, &ctx);
+            ctx.written[0] |= (1ull << var->table_id); /* Mark as written */
         }
     }
 
+    /* This function can be called multiple times when setting variables, so
+     * reset flags before setting them. */
+    it->flags &= ~(EcsIterTrivialTest|EcsIterTrivialCached|
+        EcsIterTrivialSearch);
+
+    /* Figure out whether this query can utilize specialized iterator modes for
+     * improved performance. */
     ecs_flags32_t flags = q->flags;
     ecs_query_cache_t *cache = query->cache;
-    if (flags & EcsQueryIsTrivial && !cache) {
+    if (flags & EcsQueryIsTrivial) {
         if ((flags & EcsQueryMatchOnlySelf)) {
             if (it_written) {
-                flecs_query_set_iter_this(it, ctx);
+                /* When we're testing against an entity or table, set the $this
+                 * variable in advance since it won't change later on. This 
+                 * initializes it.count, it.entities and it.table. */
+                flecs_query_set_iter_this(it, &ctx);
 
-                if (flags & EcsQueryMatchWildcards) {
-                    it->flags |= EcsIterTrivialTestWildcard;
-                    flecs_query_setids(&query->ops[0], false, ctx);
-                } else {
-                    it->flags |= EcsIterTrivialTest;
-                    flecs_query_setids(&query->ops[0], false, ctx);
+                if (!cache) {
+                    if (!(flags & EcsQueryMatchWildcards)) {
+                        it->flags |= EcsIterTrivialTest;
+                    }
+                } else if (flags & EcsQueryIsCacheable) {
+                    it->flags |= EcsIterTrivialTest|EcsIterTrivialCached;
                 }
             } else {
-                if (flags & EcsQueryMatchWildcards) {
-                    it->flags |= EcsIterTrivialSearchWildcard;
-                    flecs_query_setids(&query->ops[0], false, ctx);
-                } else if (flags & EcsQueryNoData) {
-                    it->flags |= EcsIterTrivialSearchNoData;
-                    flecs_query_setids(&query->ops[0], false, ctx);
-                } else {
-                    it->flags |= EcsIterTrivialSearch;
-                    flecs_query_setids(&query->ops[0], false, ctx);
+                if (!cache) {
+                    if (!(flags & EcsQueryMatchWildcards)) {
+                        it->flags |= EcsIterTrivialSearch;
+                    }
+                } else if (flags & EcsQueryIsCacheable) {
+                    if (!cache->order_by_callback) {
+                        it->flags |= EcsIterTrivialSearch|EcsIterTrivialCached;
+                    }
                 }
             }
+
+            /* If we're using a specialized iterator mode, make sure to 
+             * initialize static component ids. Usually this is the first 
+             * instruction of a query plan, but because we're not running the
+             * query plan when using a specialized iterator mode, manually call
+             * the operation on iterator init. */
+            flecs_query_setids(NULL, false, &ctx);
         }
     }
-
-    if (cache) {
-        cache->prev_match_count = cache->match_count;
-    }
-
-    flecs_iter_validate(it);
 }
 
-bool flecs_query_next_instanced(
+bool ecs_query_next(
     ecs_iter_t *it)
 {
     ecs_assert(it != NULL, ECS_INVALID_PARAMETER, NULL);
@@ -83,97 +110,77 @@ bool flecs_query_next_instanced(
     ecs_query_iter_t *qit = &it->priv_.iter.query;
     ecs_query_impl_t *impl = ECS_CONST_CAST(ecs_query_impl_t*, qit->query);
     ecs_query_run_ctx_t ctx;
-    ctx.world = it->real_world;
-    ctx.query = impl;
-    ctx.it = it;
-    ctx.vars = qit->vars;
-    ctx.query_vars = qit->query_vars;
-    ctx.written = qit->written;
-    ctx.op_ctx = qit->op_ctx;
-    ctx.qit = qit;
+    flecs_query_iter_run_ctx_init(it, &ctx);
     const ecs_query_op_t *ops = qit->ops;
 
-    bool redo = true;
-    if (!(it->flags & EcsIterIsValid)) {
-        ecs_assert(impl != NULL, ECS_INVALID_PARAMETER, NULL);
-        flecs_query_iter_init(&ctx);
-        redo = false;
-    } else {
-        it->frame_offset += it->count;
-
+    bool redo = it->flags & EcsIterIsValid;
+    if (redo) {
+        /* Change detection */
         if (!(it->flags & EcsIterSkip)) {
+            /* Mark table columns that are written to dirty */
             flecs_query_mark_fields_dirty(impl, it);
             if (qit->prev) {
                 if (ctx.query->pub.flags & EcsQueryHasMonitor) {
+                    /* If this query uses change detection, synchronize the
+                     * monitor for the iterated table with the query */
                     flecs_query_sync_match_monitor(impl, qit->prev);
                 }
             }
         }
-
-        it->flags &= ~EcsIterSkip;
     }
 
-    /* Specialized iterator modes for trivial queries */
-    if (it->flags & EcsIterTrivialSearch) {
-        ecs_query_trivial_ctx_t *op_ctx = &ctx.op_ctx[0].is.trivial;
-        int32_t fields = ctx.query->pub.term_count;
-        ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
-        if (!flecs_query_trivial_search(ctx.query, &ctx, op_ctx, !redo, mask)) {
-            goto done;
+    it->flags &= ~(EcsIterSkip);
+    it->flags |= EcsIterIsValid;
+    it->frame_offset += it->count;
+
+    /* Specialized iterator modes. When a query doesn't use any advanced 
+     * features, it can call specialized iterator functions directly instead of
+     * going through the dispatcher of the query engine. 
+     * The iterator mode is set during iterator initialization. Besides being
+     * determined by the query, there are different modes for searching and 
+     * testing, where searching returns all matches for a query, whereas testing
+     * tests a single table or table range against the query. */
+
+    if (it->flags & EcsIterTrivialCached) {
+        /* Cached iterator modes */
+        if (it->flags & EcsIterTrivialSearch) {
+            if (flecs_query_is_cache_search(&ctx)) {
+                goto trivial_search_yield;
+            }
+        } else if (it->flags & EcsIterTrivialTest) {
+            if (flecs_query_is_cache_test(&ctx, redo)) {
+                goto yield;
+            }
         }
-        it->table = ctx.vars[0].range.table;
-        it->count = ecs_table_count(it->table);
-        it->entities = flecs_table_entities_array(it->table);
-        return true;
-    } else if (it->flags & EcsIterTrivialSearchNoData) {
-        ecs_query_trivial_ctx_t *op_ctx = &ctx.op_ctx[0].is.trivial;
-        int32_t fields = ctx.query->pub.term_count;
-        ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
-        if (!flecs_query_trivial_search_nodata(ctx.query, &ctx, op_ctx, !redo, mask)) {
-            goto done;
+    } else {
+        /* Uncached iterator modes */
+        if (it->flags & EcsIterTrivialSearch) {
+            ecs_query_trivial_ctx_t *op_ctx = &ctx.op_ctx[0].is.trivial;
+            if (flecs_query_is_trivial_search(&ctx, op_ctx, redo)) {
+                goto yield;
+            }
+        } else if (it->flags & EcsIterTrivialTest) {
+            int32_t fields = ctx.query->pub.term_count;
+            ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
+            if (flecs_query_trivial_test(&ctx, redo, mask)) {
+                goto yield;
+            }
+        } else {
+            /* Default iterator mode. This enters the query VM dispatch loop. */
+            if (flecs_query_run_until(
+                redo, &ctx, ops, -1, qit->op, impl->op_count - 1)) 
+            {
+                ecs_assert(ops[ctx.op_index].kind == EcsQueryYield, 
+                    ECS_INTERNAL_ERROR, NULL);
+                flecs_query_set_iter_this(it, &ctx);
+                ecs_assert(it->count >= 0, ECS_INTERNAL_ERROR, NULL);
+                qit->op = flecs_itolbl(ctx.op_index - 1);
+                goto yield;
+            }
         }
-        it->table = ctx.vars[0].range.table;
-        it->count = ecs_table_count(it->table);
-        it->entities = flecs_table_entities_array(it->table);
-        return true;
-    } else if (it->flags & EcsIterTrivialSearchWildcard) {
-        ecs_query_trivial_ctx_t *op_ctx = &ctx.op_ctx[0].is.trivial;
-        int32_t fields = ctx.query->pub.term_count;
-        ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
-        if (!flecs_query_trivial_search_w_wildcards(ctx.query, &ctx, op_ctx, !redo, mask)) {
-            goto done;
-        }
-        it->table = ctx.vars[0].range.table;
-        it->count = ecs_table_count(it->table);
-        it->entities = flecs_table_entities_array(it->table);
-        return true;
-    } else if (it->flags & EcsIterTrivialTest) {
-        int32_t fields = ctx.query->pub.term_count;
-        ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
-        if (!flecs_query_trivial_test(ctx.query, &ctx, !redo, mask)) {
-            goto done;
-        }
-        return true;
-    } else if (it->flags & EcsIterTrivialTestWildcard) {
-        int32_t fields = ctx.query->pub.term_count;
-        ecs_flags64_t mask = (2llu << (fields - 1)) - 1;
-        if (!flecs_query_trivial_test_w_wildcards(ctx.query, &ctx, !redo, mask)) {
-            goto done;
-        }
-        return true;
     }
 
-    /* Default iterator mode */
-    if (flecs_query_run_until(redo, &ctx, ops, -1, qit->op, impl->op_count - 1)) {
-        ecs_assert(ops[ctx.op_index].kind == EcsQueryYield, 
-            ECS_INTERNAL_ERROR, NULL);
-        flecs_query_set_iter_this(it, &ctx);
-        ecs_assert(it->count >= 0, ECS_INTERNAL_ERROR, NULL);
-        qit->op = flecs_itolbl(ctx.op_index - 1);
-        return true;
-    }
-
-done:
+    /* Done iterating */
     flecs_query_mark_fixed_fields_dirty(impl, it);
     if (ctx.query->monitor) {
         flecs_query_update_fixed_monitor(
@@ -182,21 +189,14 @@ done:
 
     ecs_iter_fini(it);
     return false;
-}
 
-bool ecs_query_next(
-    ecs_iter_t *it)
-{
-    ecs_check(it != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(it->next == ecs_query_next, ECS_INVALID_PARAMETER, NULL);
+trivial_search_yield:
+    it->table = ctx.vars[0].range.table;
+    it->count = ecs_table_count(it->table);
+    it->entities = ecs_table_entities(it->table);
 
-    if (flecs_iter_next_row(it)) {
-        return true;
-    }
-
-    return flecs_iter_next_instanced(it, flecs_query_next_instanced(it));
-error:
-    return false;
+yield:
+    return true;
 }
 
 static
@@ -260,6 +260,7 @@ void flecs_query_iter_fini(
     flecs_iter_free_n(qit->vars, ecs_var_t, var_count);
     flecs_iter_free_n(qit->written, ecs_write_flags_t, op_count);
     flecs_iter_free_n(qit->op_ctx, ecs_query_op_ctx_t, op_count);
+
     qit->vars = NULL;
     qit->written = NULL;
     qit->op_ctx = NULL;
@@ -277,7 +278,8 @@ ecs_iter_t flecs_query_iter(
     flecs_poly_assert(q, ecs_query_t);
     ecs_query_impl_t *impl = flecs_query_impl(q);
 
-    int32_t i, var_count = impl->var_count, op_count = impl->op_count;
+    int32_t i, var_count = impl->var_count;
+    int32_t op_count = impl->op_count ? impl->op_count : 1;
     it.world = ECS_CONST_CAST(ecs_world_t*, world);
 
     /* If world passed to iterator is the real world, but query was created from
@@ -291,6 +293,10 @@ ecs_iter_t flecs_query_iter(
     it.real_world = q->real_world;
     ecs_assert(flecs_poly_is(it.real_world, ecs_world_t),
         ECS_INTERNAL_ERROR, NULL);
+    ecs_check(!(it.real_world->flags & EcsWorldMultiThreaded) || 
+        it.world != it.real_world, ECS_INVALID_PARAMETER, 
+            "create iterator for stage when world is in multithreaded mode");
+
     it.query = q;
     it.system = q->entity;
     it.next = ecs_query_next;
@@ -298,12 +304,14 @@ ecs_iter_t flecs_query_iter(
     it.field_count = q->field_count;
     it.sizes = q->sizes;
     it.set_fields = q->set_fields;
+    it.ref_fields = q->fixed_fields | q->row_fields;
+    it.row_fields = q->row_fields;
     it.up_fields = 0;
     flecs_query_apply_iter_flags(&it, q);
 
     flecs_iter_init(it.world, &it, 
         flecs_iter_cache_ids |
-        flecs_iter_cache_columns |
+        flecs_iter_cache_trs |
         flecs_iter_cache_sources |
         flecs_iter_cache_ptrs);
 
@@ -322,6 +330,8 @@ ecs_iter_t flecs_query_iter(
             qit->last = ecs_vec_last_t(
                 &cache->table_slices, ecs_query_cache_table_match_t);
         }
+
+        cache->prev_match_count = cache->match_count;
     }
 
     if (var_count) {
@@ -345,6 +355,9 @@ ecs_iter_t flecs_query_iter(
     it.variable_count = impl->pub.var_count;
     it.variable_names = impl->pub.vars;
 
+    /* Set flags for unconstrained query iteration. Can be reinitialized when
+     * variables are constrained on iterator. */
+    flecs_query_iter_constrain(&it);
 error:
     return it;
 }
