@@ -25,8 +25,7 @@ void Rest_get(void) {
     test_assert(reply_str != NULL);
     test_str(reply_str,
             "{\"parent\":\"flecs.core\", \"name\":\"World\", "
-                "\"pairs\":{\"ChildOf\":\"core\"}, "
-                "\"components\":{\"(Identifier,Name)\":null, \"(Identifier,Symbol)\":null, \"(Description,Brief)\":{\"value\":\"Entity associated with world\"}}}");
+                "\"components\":{\"(flecs.core.Identifier,flecs.core.Symbol)\":null, \"(flecs.doc.Description,flecs.doc.Brief)\":{\"value\":\"Entity associated with world\"}}}");
     ecs_os_free(reply_str);
 
     ecs_rest_server_fini(srv);
@@ -53,8 +52,7 @@ void Rest_get_cached(void) {
         test_assert(reply_str != NULL);
         test_str(reply_str,
             "{\"parent\":\"flecs.core\", \"name\":\"World\", "
-                "\"pairs\":{\"ChildOf\":\"core\"}, "
-                "\"components\":{\"(Identifier,Name)\":null, \"(Identifier,Symbol)\":null, \"(Description,Brief)\":{\"value\":\"Entity associated with world\"}}}");
+                "\"components\":{\"(flecs.core.Identifier,flecs.core.Symbol)\":null, \"(flecs.doc.Description,flecs.doc.Brief)\":{\"value\":\"Entity associated with world\"}}}");
         ecs_os_free(reply_str);
     }
 
@@ -68,8 +66,7 @@ void Rest_get_cached(void) {
         test_assert(reply_str != NULL);
         test_str(reply_str,
             "{\"parent\":\"flecs.core\", \"name\":\"World\", "
-                "\"pairs\":{\"ChildOf\":\"core\"}, "
-                "\"components\":{\"(Identifier,Name)\":null, \"(Identifier,Symbol)\":null, \"(Description,Brief)\":{\"value\":\"Entity associated with world\"}}}");
+                "\"components\":{\"(flecs.core.Identifier,flecs.core.Symbol)\":null, \"(flecs.doc.Description,flecs.doc.Brief)\":{\"value\":\"Entity associated with world\"}}}");
         ecs_os_free(reply_str);
     }
 
