@@ -409,6 +409,32 @@ void Commands_system_in_progress_w_defer(void) {
     ecs_fini(world);
 }
 
+void Commands_defer_ensure(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_entity_t e = ecs_new(world);
+
+    ecs_defer_begin(world);
+    {
+        Position *p = ecs_ensure(world, e, Position);
+        test_assert(p != NULL);
+        p->x = 10;
+        p->y = 20;
+    }
+    ecs_defer_end(world);
+
+    {
+        Position *p = ecs_get_mut(world, e, Position);
+        test_assert(p != NULL);
+        test_int(p->x, 10);
+        test_int(p->y, 20);
+    }
+
+    ecs_fini(world);
+}
+
 static bool on_set_invoked = 0;
 
 static
@@ -1766,6 +1792,61 @@ void Commands_defer_disable(void) {
     ecs_fini(world);
 }
 
+void Commands_defer_enable_from_stage(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ecs_entity_t e = ecs_new(world);
+
+    ecs_world_t *stage = ecs_get_stage(world, 0);
+
+    ecs_defer_begin(world);
+    ecs_enable(stage, e, false);
+    test_assert(!ecs_has_id(world, e, EcsDisabled));
+    ecs_defer_end(world);
+
+    test_assert(ecs_has_id(world, e, EcsDisabled));
+
+    ecs_fini(world);
+}
+
+void Commands_defer_toggle(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Foo);
+    ecs_add_id(world, Foo, EcsCanToggle);
+
+    ecs_entity_t e = ecs_new_w(world, Foo);
+
+    ecs_defer_begin(world);
+    ecs_enable_id(world, e, Foo, false);
+    test_assert(ecs_is_enabled_id(world, e, Foo));
+    ecs_defer_end(world);
+
+    test_assert(!ecs_is_enabled_id(world, e, Foo));
+
+    ecs_fini(world);
+}
+
+void Commands_defer_toggle_from_stage(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Foo);
+    ecs_add_id(world, Foo, EcsCanToggle);
+
+    ecs_entity_t e = ecs_new_w(world, Foo);
+
+    ecs_world_t *stage = ecs_get_stage(world, 0);
+
+    ecs_defer_begin(world);
+    ecs_enable_id(stage, e, Foo, false);
+    test_assert(ecs_is_enabled_id(stage, e, Foo));
+    ecs_defer_end(world);
+
+    test_assert(!ecs_is_enabled_id(world, e, Foo));
+
+    ecs_fini(world);
+}
+
 void Commands_defer_delete_with(void) {
     ecs_world_t *world = ecs_mini();
 
@@ -4044,3 +4125,31 @@ void Commands_add_batched_set_with(void) {
 
     ecs_fini(world);
 }
+
+void Commands_defer_emplace_after_remove(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_entity_t e = ecs_insert(world, ecs_value(Position, {0, 0}));
+
+    ecs_defer_begin(world);
+    ecs_remove(world, e, Position);
+    {
+        Position *p = ecs_emplace(world, e, Position, NULL);
+        test_assert(p != NULL);
+        p->x = 10;
+        p->y = 20;
+    }
+    ecs_defer_end(world);
+    
+    test_assert(ecs_has(world, e, Position));
+    {
+        const Position *p = ecs_get(world, e, Position);
+        test_assert(p != NULL);
+        test_int(p->x, 10);
+        test_int(p->y, 20);
+    }
+
+    ecs_fini(world);
+}
