@@ -778,6 +778,26 @@ void Observer_on_set_w_defer_set(void) {
     test_int(count, 1);
 }
 
+void Observer_on_set_w_set_sparse(void) {
+    flecs::world world;
+
+    world.component<Position>().add(flecs::Sparse);
+
+    int32_t count = 0;
+
+    world.observer<Position>()
+        .event(flecs::OnSet)
+        .each([&](flecs::entity e, Position& p) {
+            count ++;          
+        });
+
+    flecs::entity e = world.entity();
+    test_int(count, 0);
+
+    e.set<Position>({10, 20});
+    test_int(count, 1);
+}
+
 #include <iostream>
 
 void Observer_on_add_singleton(void) {
@@ -943,10 +963,10 @@ void Observer_term_index(void) {
         });
 
     e1.set<Position>({ 10, 20 });
-    test_int(last_term, 1);
+    test_int(last_term, 0);
 
     e1.set<Velocity>({ 30, 40 }); 
-    test_int(last_term, 2);
+    test_int(last_term, 1);
 }
 
 void Observer_implicit_register_in_emit_for_named_entity(void) {
@@ -1020,3 +1040,99 @@ void Observer_add_to_named_in_emit_for_named_entity_w_defer(void) {
     e1.emit<MyEvent>({ 10 });
     world.defer_end();
 }
+
+void Observer_register_twice_w_each(void) {
+    flecs::world ecs;
+
+    int count1 = 0, count2 = 0;
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .each([&](Position&) {
+            count1 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count1, 1);
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .each([&](Position&) {
+            count2 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count2, 1);
+}
+
+void Observer_register_twice_w_run(void) {
+    flecs::world ecs;
+
+    int count1 = 0, count2 = 0;
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .run([&](flecs::iter&) {
+            count1 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count1, 1);
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .run([&](flecs::iter&) {
+            count2 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count2, 1);
+}
+
+void Observer_register_twice_w_run_each(void) {
+    flecs::world ecs;
+
+    int count1 = 0, count2 = 0;
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .run([&](flecs::iter&) {
+            count1 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count1, 1);
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .each([&](Position&) {
+            count2 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count2, 1);
+}
+
+void Observer_register_twice_w_each_run(void) {
+    flecs::world ecs;
+
+    int count1 = 0, count2 = 0;
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .each([&](Position&) {
+            count1 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count1, 1);
+
+    ecs.observer<Position>("Test")
+        .event(flecs::OnSet)
+        .run([&](flecs::iter&) {
+            count2 ++;
+        });
+
+    ecs.entity().set(Position{10, 20});
+    test_int(count2, 1);
+}
