@@ -599,7 +599,7 @@ void Query_optional_pair_term(void) {
     test_int(1, without_pair);
 }
 
-void Query_action(void) {
+void Query_run(void) {
     flecs::world world;
 
     world.component<Position>();
@@ -628,7 +628,7 @@ void Query_action(void) {
     test_int(p->y, 22);
 }
 
-void Query_action_const(void) {
+void Query_run_const(void) {
     flecs::world world;
 
     world.component<Position>();
@@ -657,7 +657,7 @@ void Query_action_const(void) {
     test_int(p->y, 22);
 }
 
-void Query_action_shared(void) {
+void Query_run_shared(void) {
     flecs::world world;
 
     world.component<Position>().add(flecs::OnInstantiate, flecs::Inherit);
@@ -706,7 +706,7 @@ void Query_action_shared(void) {
     test_int(p->y, 24);  
 }
 
-void Query_action_optional(void) {
+void Query_run_optional(void) {
     flecs::world world;
 
     world.component<Position>();
@@ -768,6 +768,35 @@ void Query_action_optional(void) {
     test_int(p->y, 81);
 }
 
+void Query_run_sparse(void) {
+    flecs::world world;
+
+    world.component<Position>().add(flecs::Sparse);
+    world.component<Velocity>();
+
+    auto entity = flecs::entity(world)
+        .set<Position>({10, 20})
+        .set<Velocity>({1, 2});
+
+    auto q = world.query<Position, Velocity>();
+
+    q.run([](flecs::iter& it) {
+        while (it.next()) {
+            auto v = it.field<Velocity>(1);
+
+            for (auto i : it) {
+                auto& p = it.field_at<Position>(0, i);
+                p.x += v[i].x;
+                p.y += v[i].y;
+            }
+        }
+    });
+
+    const Position *p = entity.get<Position>();
+    test_int(p->x, 11);
+    test_int(p->y, 22);
+}
+
 void Query_each(void) {
     flecs::world world;
 
@@ -905,6 +934,28 @@ void Query_each_optional(void) {
     test_int(p->y, 81);  
 }
 
+void Query_each_sparse(void) {
+    flecs::world world;
+
+    world.component<Position>().add(flecs::Sparse);
+    world.component<Velocity>();
+
+    auto entity = flecs::entity(world)
+        .set<Position>({10, 20})
+        .set<Velocity>({1, 2});
+
+    auto q = world.query<Position, Velocity>();
+
+    q.each([](Position& p, Velocity& v) {
+        p.x += v.x;
+        p.y += v.y;
+    });
+
+    const Position *p = entity.get<Position>();
+    test_int(p->x, 11);
+    test_int(p->y, 22);
+}
+
 // Generic lambdas are a C++14 feature.
 
 struct GenericLambdaEachEntity {
@@ -1818,7 +1869,6 @@ void Query_instanced_query_w_singleton_each(void) {
 
     auto q = ecs.query_builder<Self, Position, const Velocity>()
         .term_at(2).singleton()
-        .instanced()
         .build();
 
     int32_t count = 0;
@@ -1870,123 +1920,6 @@ void Query_instanced_query_w_base_each(void) {
     auto e6 = ecs.entity().set<Position>({60, 70}).set<Velocity>({2, 3}); e6.set<Self>({e6});
     auto e7 = ecs.entity().set<Position>({70, 80}).set<Velocity>({4, 5}); e7.set<Self>({e7});
 
-    auto q = ecs.query_builder<Self, Position, const Velocity>()
-        .instanced()
-        .build();
-
-    int32_t count = 0;
-    q.each([&](flecs::entity e, Self& s, Position&p, const Velocity& v) {
-        test_assert(e == s.value);
-        p.x += v.x;
-        p.y += v.y;
-        count ++;
-    });
-
-    test_int(count, 7);
-
-    test_assert(e1.get([](const Position& p) {
-        test_int(p.x, 11);
-        test_int(p.y, 22);
-    }));
-
-    test_assert(e2.get([](const Position& p) {
-        test_int(p.x, 21);
-        test_int(p.y, 32);
-    }));
-
-    test_assert(e3.get([](const Position& p) {
-        test_int(p.x, 31);
-        test_int(p.y, 42);
-    }));
-
-    test_assert(e4.get([](const Position& p) {
-        test_int(p.x, 41);
-        test_int(p.y, 52);
-    }));
-
-    test_assert(e5.get([](const Position& p) {
-        test_int(p.x, 51);
-        test_int(p.y, 62);
-    }));
-
-    test_assert(e6.get([](const Position& p) {
-        test_int(p.x, 62);
-        test_int(p.y, 73);
-    }));
-
-    test_assert(e7.get([](const Position& p) {
-        test_int(p.x, 74);
-        test_int(p.y, 85);
-    }));
-}
-
-void Query_un_instanced_query_w_singleton_each(void) {
-    flecs::world ecs;
-
-    ecs.set<Velocity>({1, 2});
-
-    auto e1 = ecs.entity().set<Position>({10, 20}); e1.set<Self>({e1});
-    auto e2 = ecs.entity().set<Position>({20, 30}); e2.set<Self>({e2});
-    auto e3 = ecs.entity().set<Position>({30, 40}); e3.set<Self>({e3});
-    auto e4 = ecs.entity().set<Position>({40, 50}); e4.set<Self>({e4});
-    auto e5 = ecs.entity().set<Position>({50, 60}); e5.set<Self>({e5});
-
-    e4.add<Tag>();
-    e5.add<Tag>();
-
-    auto q = ecs.query_builder<Self, Position, const Velocity>()
-        .term_at(2).singleton()
-        .build();
-
-    int32_t count = 0;
-    q.each([&](flecs::entity e, Self& s, Position&p, const Velocity& v) {
-        test_assert(e == s.value);
-        p.x += v.x;
-        p.y += v.y;
-        count ++;
-    });
-
-    test_int(count, 5);
-
-    test_assert(e1.get([](const Position& p) {
-        test_int(p.x, 11);
-        test_int(p.y, 22);
-    }));
-
-    test_assert(e2.get([](const Position& p) {
-        test_int(p.x, 21);
-        test_int(p.y, 32);
-    }));
-
-    test_assert(e3.get([](const Position& p) {
-        test_int(p.x, 31);
-        test_int(p.y, 42);
-    }));
-
-    test_assert(e4.get([](const Position& p) {
-        test_int(p.x, 41);
-        test_int(p.y, 52);
-    }));
-
-    test_assert(e5.get([](const Position& p) {
-        test_int(p.x, 51);
-        test_int(p.y, 62);
-    }));
-}
-
-void Query_un_instanced_query_w_base_each(void) {
-    flecs::world ecs;
-
-    auto base = ecs.entity().set<Velocity>({1, 2});
-
-    auto e1 = ecs.entity().is_a(base).set<Position>({10, 20}); e1.set<Self>({e1});
-    auto e2 = ecs.entity().is_a(base).set<Position>({20, 30}); e2.set<Self>({e2});
-    auto e3 = ecs.entity().is_a(base).set<Position>({30, 40}); e3.set<Self>({e3});
-    auto e4 = ecs.entity().is_a(base).set<Position>({40, 50}).add<Tag>(); e4.set<Self>({e4});
-    auto e5 = ecs.entity().is_a(base).set<Position>({50, 60}).add<Tag>(); e5.set<Self>({e5});
-    auto e6 = ecs.entity().set<Position>({60, 70}).set<Velocity>({2, 3}); e6.set<Self>({e6});
-    auto e7 = ecs.entity().set<Position>({70, 80}).set<Velocity>({4, 5}); e7.set<Self>({e7});
-
     auto q = ecs.query_builder<Self, Position, const Velocity>()
         .build();
 
@@ -2052,7 +1985,6 @@ void Query_instanced_query_w_singleton_iter(void) {
 
     auto q = ecs.query_builder<Self, Position, const Velocity>()
         .term_at(2).singleton()
-        .instanced()
         .build();
 
     int32_t count = 0;
@@ -2115,7 +2047,6 @@ void Query_instanced_query_w_base_iter(void) {
     auto e7 = ecs.entity().set<Position>({70, 80}).set<Velocity>({4, 5}); e7.set<Self>({e7});
 
     auto q = ecs.query_builder<Self, Position, const Velocity>()
-        .instanced()
         .build();
 
     int32_t count = 0;
@@ -2179,139 +2110,6 @@ void Query_instanced_query_w_base_iter(void) {
     }));
 }
 
-void Query_un_instanced_query_w_singleton_iter(void) {
-    flecs::world ecs;
-
-    ecs.set<Velocity>({1, 2});
-
-    auto e1 = ecs.entity().set<Position>({10, 20}); e1.set<Self>({e1});
-    auto e2 = ecs.entity().set<Position>({20, 30}); e2.set<Self>({e2});
-    auto e3 = ecs.entity().set<Position>({30, 40}); e3.set<Self>({e3});
-    auto e4 = ecs.entity().set<Position>({40, 50}); e4.set<Self>({e4});
-    auto e5 = ecs.entity().set<Position>({50, 60}); e5.set<Self>({e5});
-
-    e4.add<Tag>();
-    e5.add<Tag>();
-
-    auto q = ecs.query_builder<Self, Position, const Velocity>()
-        .term_at(2).singleton()
-        .build();
-
-    int32_t count = 0;
-    q.run([&](flecs::iter it) {
-        while (it.next()) {
-            auto s = it.field<Self>(0);
-            auto p = it.field<Position>(1);
-            auto v = it.field<const Velocity>(2);
-
-            test_assert(it.count() == 1);
-            for (auto i : it) {
-                p[i].x += v[i].x;
-                p[i].y += v[i].y;
-                test_assert(it.entity(i) == s[i].value);
-                count ++;
-            }
-        }
-    });
-
-    test_int(count, 5);
-
-    test_assert(e1.get([](const Position& p) {
-        test_int(p.x, 11);
-        test_int(p.y, 22);
-    }));
-
-    test_assert(e2.get([](const Position& p) {
-        test_int(p.x, 21);
-        test_int(p.y, 32);
-    }));
-
-    test_assert(e3.get([](const Position& p) {
-        test_int(p.x, 31);
-        test_int(p.y, 42);
-    }));
-
-    test_assert(e4.get([](const Position& p) {
-        test_int(p.x, 41);
-        test_int(p.y, 52);
-    }));
-
-    test_assert(e5.get([](const Position& p) {
-        test_int(p.x, 51);
-        test_int(p.y, 62);
-    }));
-}
-
-void Query_un_instanced_query_w_base_iter(void) {
-    flecs::world ecs;
-
-    auto base = ecs.entity().set<Velocity>({1, 2});
-
-    auto e1 = ecs.entity().is_a(base).set<Position>({10, 20}); e1.set<Self>({e1});
-    auto e2 = ecs.entity().is_a(base).set<Position>({20, 30}); e2.set<Self>({e2});
-    auto e3 = ecs.entity().is_a(base).set<Position>({30, 40}); e3.set<Self>({e3});
-    auto e4 = ecs.entity().is_a(base).set<Position>({40, 50}).add<Tag>(); e4.set<Self>({e4});
-    auto e5 = ecs.entity().is_a(base).set<Position>({50, 60}).add<Tag>(); e5.set<Self>({e5});
-    auto e6 = ecs.entity().set<Position>({60, 70}).set<Velocity>({2, 3}); e6.set<Self>({e6});
-    auto e7 = ecs.entity().set<Position>({70, 80}).set<Velocity>({4, 5}); e7.set<Self>({e7});
-
-    auto q = ecs.query_builder<Self, Position, const Velocity>()
-        .build();
-
-    int32_t count = 0;
-    q.run([&](flecs::iter it) {
-        while (it.next()) {
-            auto s = it.field<Self>(0);
-            auto p = it.field<Position>(1);
-            auto v = it.field<const Velocity>(2);
-
-            for (auto i : it) {
-                p[i].x += v[i].x;
-                p[i].y += v[i].y;
-                test_assert(it.entity(i) == s[i].value);
-                count ++;
-            }
-        }
-    });
-
-    test_int(count, 7);
-
-    test_assert(e1.get([](const Position& p) {
-        test_int(p.x, 11);
-        test_int(p.y, 22);
-    }));
-
-    test_assert(e2.get([](const Position& p) {
-        test_int(p.x, 21);
-        test_int(p.y, 32);
-    }));
-
-    test_assert(e3.get([](const Position& p) {
-        test_int(p.x, 31);
-        test_int(p.y, 42);
-    }));
-
-    test_assert(e4.get([](const Position& p) {
-        test_int(p.x, 41);
-        test_int(p.y, 52);
-    }));
-
-    test_assert(e5.get([](const Position& p) {
-        test_int(p.x, 51);
-        test_int(p.y, 62);
-    }));
-
-    test_assert(e6.get([](const Position& p) {
-        test_int(p.x, 62);
-        test_int(p.y, 73);
-    }));
-
-    test_assert(e7.get([](const Position& p) {
-        test_int(p.x, 74);
-        test_int(p.y, 85);
-    }));
-}
-
 void Query_query_each_from_component(void) {
     flecs::world w;
 
@@ -2789,7 +2587,7 @@ void Query_instanced_nested_query_w_iter(void) {
         while (it_2.next()) {
             q1.iter(it_2).run([&](flecs::iter& it_1) {
                 while (it_1.next()) {
-                    test_int(it_1.count(), 1);
+                    test_int(it_1.count(), 2);
                     count += it_1.count();
                 }
             });
@@ -2821,7 +2619,7 @@ void Query_instanced_nested_query_w_entity(void) {
     q2.each([&](flecs::entity e_2) {
         q1.iter(e_2).run([&](flecs::iter& it_1) {
             while (it_1.next()) {
-                test_int(it_1.count(), 1);
+                test_int(it_1.count(), 2);
                 count += it_1.count();
             }
         });
@@ -2853,7 +2651,7 @@ void Query_instanced_nested_query_w_world(void) {
         while (it_2.next()) {
             q1.iter(it_2.world()).run([&](flecs::iter& it_1) {
                 while (it_1.next()) {
-                    test_int(it_1.count(), 1);
+                    test_int(it_1.count(), 2);
                     count += it_1.count();
                 }
             });
@@ -3077,6 +2875,20 @@ void Query_run_w_iter_fini_empty(void) {
     test_int(count, 1);
 }
 
+void Query_run_w_iter_fini_no_query(void) {
+    flecs::world ecs;
+
+    flecs::query<> q = ecs.query();
+
+    int32_t count = 0;
+    q.run([&](flecs::iter& it) {
+        count ++;
+        it.fini();
+    });
+
+    test_int(count, 1);
+}
+
 void Query_add_to_match_from_staged_query(void) {
     flecs::world ecs;
 
