@@ -268,14 +268,8 @@ void Plan_1_trivial_plan(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  andid       $[this]           (Foo)"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -297,14 +291,8 @@ void Plan_2_trivial_plan(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  triv        {0,1}"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -326,15 +314,8 @@ void Plan_1_trivial_plan_component(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  andid       $[this]           (Position)"
-    LINE " 2. [ 1,  3]  popself     {0}"
-    LINE " 3. [ 2,  4]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -358,14 +339,8 @@ void Plan_2_trivial_plan_component(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivpop     {0,1}"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -389,14 +364,8 @@ void Plan_3_trivial_plan_w_pair(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  triv        {0,1,2}"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -421,9 +390,10 @@ void Plan_3_trivial_plan_w_wildcard(void) {
     ecs_log_enable_colors(false);
 
     const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivwc      {0,1,2}"
-    LINE " 2. [ 1,  3]  yield       "
+    HEAD " 0. [-1,  1]  setids      "
+    LINE " 1. [ 0,  2]  triv        {0,1}"
+    LINE " 2. [ 1,  3]  and         $[this]           (ChildOf, $*'1)"
+    LINE " 3. [ 2,  4]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -485,14 +455,8 @@ void Plan_3_trivial_plan_w_pair_component(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivpop     {0,1,2}"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -519,9 +483,10 @@ void Plan_3_trivial_plan_w_wildcard_component(void) {
     ecs_log_enable_colors(false);
 
     const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivwc      {0,1,2}"
-    LINE " 2. [ 1,  3]  yield       "
+    HEAD " 0. [-1,  1]  setids      "
+    LINE " 1. [ 0,  2]  triv        {0,1}"
+    LINE " 2. [ 1,  3]  and         $[this]           (ChildOf, $*'1)"
+    LINE " 3. [ 2,  4]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -553,7 +518,7 @@ void Plan_3_trivial_plan_w_any_component(void) {
 
     const char *expect = 
     HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivpop     {0,1}"
+    LINE " 1. [ 0,  2]  triv        {0,1}"
     LINE " 2. [ 1,  3]  andany      $[this]           (ChildOf, $_'1)"
     LINE " 3. [ 2,  4]  yield       "
     LINE "";
@@ -585,14 +550,8 @@ void Plan_1_trivial_component_w_none(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  andid       $[this]           (Position)"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -618,14 +577,8 @@ void Plan_2_trivial_component_w_none(void) {
 
     ecs_log_enable_colors(false);
 
-    const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  triv        {0,1}"
-    LINE " 2. [ 1,  3]  yield       "
-    LINE "";
     char *plan = ecs_query_plan(r);
-
-    test_str(expect, plan);
+    test_str(NULL, plan);
     ecs_os_free(plan);
 
     ecs_query_fini(r);
@@ -647,9 +600,10 @@ void Plan_2_trivial_plan_w_wildcard(void) {
     ecs_log_enable_colors(false);
 
     const char *expect = 
-    HEAD " 0. [-1,  1]  setids      " 
-    LINE " 1. [ 0,  2]  trivwc      {0,1}"
-    LINE " 2. [ 1,  3]  yield       "
+    HEAD " 0. [-1,  1]  setids      "
+    LINE " 1. [ 0,  2]  andid       $[this]           (Foo)"
+    LINE " 2. [ 1,  3]  and         $[this]           (ChildOf, $*'1)"
+    LINE " 3. [ 2,  4]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -996,8 +950,7 @@ void Plan_populate_1_fixed(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  yield       "
+    LINE " 3. [ 2,  4]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1032,8 +985,7 @@ void Plan_populate_2_fixed(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  yield       "
+    LINE " 4. [ 3,  5]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1067,10 +1019,8 @@ void Plan_populate_1_fixed_1_this_self(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  andid       $[this]           (Velocity)"
-    LINE " 5. [ 4,  6]  popself     {1}"
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 3. [ 2,  4]  andid       $[this]           (Velocity)"
+    LINE " 4. [ 3,  5]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1109,9 +1059,8 @@ void Plan_populate_2_fixed_2_this_self(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  trivpop     {2,3}"
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  triv        {2,3}"
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1150,9 +1099,8 @@ void Plan_populate_2_fixed_2_this_self_interleaved(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  trivpop     {1,3}"
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  triv        {1,3}"
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1191,9 +1139,8 @@ void Plan_populate_2_this_self_2_fixed(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  trivpop     {0,1}"
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  triv        {0,1}"
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1227,10 +1174,8 @@ void Plan_populate_1_fixed_1_this_up(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  upid        $[this]           (Velocity)"
-    LINE " 5. [ 4,  6]  pop         {1}"
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 3. [ 2,  4]  upid        $[this]           (Velocity)"
+    LINE " 4. [ 3,  5]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1269,11 +1214,9 @@ void Plan_populate_2_fixed_2_this_up(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  upid        $[this]           (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[this]           (Rotation)"
-    LINE " 7. [ 6,  8]  pop         {2,3}"
-    LINE " 8. [ 7,  9]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[this]           (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[this]           (Rotation)"
+    LINE " 6. [ 5,  7]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1312,11 +1255,9 @@ void Plan_populate_2_fixed_2_this_up_interleaved(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  upid        $[this]           (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[this]           (Rotation)"
-    LINE " 7. [ 6,  8]  pop         {1,3}"
-    LINE " 8. [ 7,  9]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[this]           (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[this]           (Rotation)"
+    LINE " 6. [ 5,  7]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1355,11 +1296,9 @@ void Plan_populate_2_this_up_2_fixed(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  upid        $[this]           (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[this]           (Rotation)"
-    LINE " 7. [ 6,  8]  pop         {0,1}"
-    LINE " 8. [ 7,  9]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[this]           (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[this]           (Rotation)"
+    LINE " 6. [ 5,  7]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1394,9 +1333,8 @@ void Plan_populate_1_fixed_1_this_self_cached(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  cachepop    "
-    LINE " 5. [ 4,  6]  yield       "
+    LINE " 3. [ 2,  4]  cache       "
+    LINE " 4. [ 3,  5]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1436,9 +1374,8 @@ void Plan_populate_2_fixed_2_this_self_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1478,9 +1415,8 @@ void Plan_populate_2_fixed_2_this_self_interleaved_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1520,9 +1456,8 @@ void Plan_populate_2_this_self_2_fixed_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1557,9 +1492,8 @@ void Plan_populate_1_fixed_1_this_up_cached(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  cachepop    "
-    LINE " 5. [ 4,  6]  yield       "
+    LINE " 3. [ 2,  4]  cache       "
+    LINE " 4. [ 3,  5]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1599,9 +1533,8 @@ void Plan_populate_2_fixed_2_this_up_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1641,9 +1574,8 @@ void Plan_populate_2_fixed_2_this_up_interleaved_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1683,9 +1615,8 @@ void Plan_populate_2_this_up_2_fixed_cached(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  cachepop    "
-    LINE " 6. [ 5,  7]  yield       "
+    LINE " 4. [ 3,  5]  cache       "
+    LINE " 5. [ 4,  6]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1720,12 +1651,10 @@ void Plan_populate_1_fixed_1_var_self(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  andid       $[var]            (Velocity)"
-    LINE " 5. [ 4,  6]  each        $var              ($[var])"
-    LINE " 6. [ 5,  7]  setvars     "
-    LINE " 7. [ 6,  8]  pop         {1}"
-    LINE " 8. [ 7,  9]  yield       "
+    LINE " 3. [ 2,  4]  andid       $[var]            (Velocity)"
+    LINE " 4. [ 3,  5]  each        $var              ($[var])"
+    LINE " 5. [ 4,  6]  setvars     "
+    LINE " 6. [ 5,  7]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1765,13 +1694,11 @@ void Plan_populate_2_fixed_2_var_self(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  andid       $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  andid       $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {2,3}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  andid       $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  andid       $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1811,13 +1738,11 @@ void Plan_populate_2_fixed_2_var_self_interleaved(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  andid       $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  andid       $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {1,3}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  andid       $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  andid       $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1857,13 +1782,11 @@ void Plan_populate_2_var_self_2_fixed(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  andid       $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  andid       $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {0,1}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  andid       $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  andid       $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1898,12 +1821,10 @@ void Plan_populate_1_fixed_1_var_up(void) {
     HEAD " 0. [-1,  1]  setfix      "
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
-    LINE " 3. [ 2,  4]  pop         {0}"
-    LINE " 4. [ 3,  5]  upid        $[var]            (Velocity)"
-    LINE " 5. [ 4,  6]  each        $var              ($[var])"
-    LINE " 6. [ 5,  7]  setvars     "
-    LINE " 7. [ 6,  8]  pop         {1}"
-    LINE " 8. [ 7,  9]  yield       "
+    LINE " 3. [ 2,  4]  upid        $[var]            (Velocity)"
+    LINE " 4. [ 3,  5]  each        $var              ($[var])"
+    LINE " 5. [ 4,  6]  setvars     "
+    LINE " 6. [ 5,  7]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1943,13 +1864,11 @@ void Plan_populate_2_fixed_2_var_up(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,1}"
-    LINE " 5. [ 4,  6]  upid        $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {2,3}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -1989,13 +1908,11 @@ void Plan_populate_2_fixed_2_var_up_interleaved(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {0,2}"
-    LINE " 5. [ 4,  6]  upid        $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {1,3}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -2035,13 +1952,11 @@ void Plan_populate_2_var_up_2_fixed(void) {
     LINE " 1. [ 0,  2]  setids      "
     LINE " 2. [ 1,  3]  selfupid    e                 (Position)"
     LINE " 3. [ 2,  4]  selfupid    e                 (Velocity)"
-    LINE " 4. [ 3,  5]  pop         {2,3}"
-    LINE " 5. [ 4,  6]  upid        $[var]            (Mass)"
-    LINE " 6. [ 5,  7]  upid        $[var]            (Rotation)"
-    LINE " 7. [ 6,  8]  each        $var              ($[var])"
-    LINE " 8. [ 7,  9]  setvars     "
-    LINE " 9. [ 8, 10]  pop         {0,1}"
-    LINE "10. [ 9, 11]  yield       "
+    LINE " 4. [ 3,  5]  upid        $[var]            (Mass)"
+    LINE " 5. [ 4,  6]  upid        $[var]            (Rotation)"
+    LINE " 6. [ 5,  7]  each        $var              ($[var])"
+    LINE " 7. [ 6,  8]  setvars     "
+    LINE " 8. [ 7,  9]  yield       "
     LINE "";
     char *plan = ecs_query_plan(r);
 
@@ -2317,3 +2232,245 @@ void Plan_pair_second_wildcard_cached(void) {
 
     ecs_fini(world);
 }
+
+void Plan_0_src_tag(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Foo);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Foo(#0)"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Plan_0_src_component(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0)"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Plan_0_src_w_sparse(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0)"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Plan_0_src_w_toggle(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsCanToggle);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0)"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Plan_0_src_w_union(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Movement);
+
+    ecs_add_id(world, ecs_id(Movement), EcsUnion);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Movement(#0, *)"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Plan_0_src_w_sparse_and_component(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0), Velocity"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  andid       $[this]           (Velocity)"
+    LINE " 3. [ 2,  4]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world); 
+}
+
+void Plan_0_src_w_toggle_and_component(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_add_id(world, ecs_id(Position), EcsCanToggle);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0), Velocity"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  andid       $[this]           (Velocity)"
+    LINE " 3. [ 2,  4]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world); 
+}
+
+void Plan_0_src_w_union_and_component(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Movement);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_add_id(world, ecs_id(Movement), EcsUnion);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Movement(#0, *), Velocity"
+    });
+
+    test_assert(q != NULL);
+
+    ecs_log_enable_colors(false);
+
+    const char *expect = 
+    HEAD " 0. [-1,  1]  setfix      "
+    LINE " 1. [ 0,  2]  setids      "
+    LINE " 2. [ 1,  3]  andid       $[this]           (Velocity)"
+    LINE " 3. [ 2,  4]  yield       "
+    LINE "";
+    char *plan = ecs_query_plan(q);
+
+    test_str(expect, plan);
+    ecs_os_free(plan);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
