@@ -31,12 +31,14 @@ void Sparse_1_fixed_sparse(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(ent, ecs_field_src(&it, 0));
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
@@ -64,14 +66,12 @@ void Sparse_1_fixed_sparse_none(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(ent, ecs_field_src(&it, 0));
-    {
-        Position *p = ecs_field(&it, Position, 0);
-        test_assert(p == NULL);
-    }
 
     test_bool(false, ecs_query_next(&it));
 
@@ -93,34 +93,32 @@ void Sparse_1_this_sparse(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     ecs_entity_t e1 = ecs_insert(world, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, ecs_value(Position, {30, 40}));
     ecs_entity_t e3 = ecs_insert(world, ecs_value(Position, {50, 60}));
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(3, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e3, it.entities[0]);
+    test_uint(e3, it.entities[2]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 2);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
@@ -145,6 +143,8 @@ void Sparse_1_this_sparse_none(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+
     ecs_entity_t e1 = ecs_insert(world, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, ecs_value(Position, {30, 40}));
     ecs_entity_t e3 = ecs_insert(world, ecs_value(Position, {50, 60}));
@@ -155,8 +155,6 @@ void Sparse_1_this_sparse_none(void) {
     test_uint(e1, it.entities[0]);
     test_uint(e2, it.entities[1]);
     test_uint(e3, it.entities[2]);
-    Position *p = ecs_field(&it, Position, 0);
-    test_assert(p == NULL);
     test_bool(false, ecs_query_next(&it));
 
     ecs_query_fini(q);
@@ -177,6 +175,9 @@ void Sparse_1_this_sparse_written(void) {
         .cache_kind = cache_kind
     });
     test_assert(q != NULL);
+    
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
 
     ecs_entity_t e1 = ecs_insert(world, {Foo}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {Foo}, ecs_value(Position, {30, 40}));
@@ -184,28 +185,24 @@ void Sparse_1_this_sparse_written(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(3, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e3, it.entities[0]);
+    test_uint(e3, it.entities[2]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 2);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
@@ -231,6 +228,9 @@ void Sparse_1_this_sparse_written_none(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(false, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t e1 = ecs_insert(world, {Foo}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {Foo}, ecs_value(Position, {30, 40}));
     ecs_entity_t e3 = ecs_insert(world, {Foo}, ecs_value(Position, {50, 60}));
@@ -241,8 +241,6 @@ void Sparse_1_this_sparse_written_none(void) {
     test_uint(e1, it.entities[0]);
     test_uint(e2, it.entities[1]);
     test_uint(e3, it.entities[2]);
-    Position *p = ecs_field(&it, Position, 1);
-    test_assert(p == NULL);
     test_bool(false, ecs_query_next(&it));
 
     ecs_query_fini(q);
@@ -263,6 +261,8 @@ void Sparse_1_var_sparse(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     int x_var = ecs_query_find_var(q, "x");
     test_assert(x_var != -1);
 
@@ -275,7 +275,7 @@ void Sparse_1_var_sparse(void) {
     test_int(0, it.count);
     test_uint(e1, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
@@ -284,7 +284,7 @@ void Sparse_1_var_sparse(void) {
     test_int(0, it.count);
     test_uint(e2, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
@@ -293,7 +293,7 @@ void Sparse_1_var_sparse(void) {
     test_int(0, it.count);
     test_uint(e3, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
@@ -318,6 +318,8 @@ void Sparse_1_var_sparse_none(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+
     int x_var = ecs_query_find_var(q, "x");
     test_assert(x_var != -1);
 
@@ -329,26 +331,14 @@ void Sparse_1_var_sparse_none(void) {
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e1, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 0);
-        test_assert(p == NULL);
-    }
 
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e2, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 0);
-        test_assert(p == NULL);
-    }
 
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e3, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 0);
-        test_assert(p == NULL);
-    }
 
     test_bool(false, ecs_query_next(&it));
 
@@ -371,6 +361,9 @@ void Sparse_1_var_sparse_written(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     int x_var = ecs_query_find_var(q, "x");
     test_assert(x_var != -1);
 
@@ -383,7 +376,7 @@ void Sparse_1_var_sparse_written(void) {
     test_int(0, it.count);
     test_uint(e1, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
@@ -392,7 +385,7 @@ void Sparse_1_var_sparse_written(void) {
     test_int(0, it.count);
     test_uint(e2, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
@@ -401,7 +394,7 @@ void Sparse_1_var_sparse_written(void) {
     test_int(0, it.count);
     test_uint(e3, ecs_iter_get_var(&it, x_var));
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
@@ -427,6 +420,9 @@ void Sparse_1_var_sparse_written_none(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(false, !!(q->row_fields & (1llu << 1)));
+
     int x_var = ecs_query_find_var(q, "x");
     test_assert(x_var != -1);
 
@@ -438,26 +434,14 @@ void Sparse_1_var_sparse_written_none(void) {
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e1, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 1);
-        test_assert(p == NULL);
-    }
 
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e2, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 1);
-        test_assert(p == NULL);
-    }
 
     test_bool(true, ecs_query_next(&it));
     test_int(0, it.count);
     test_uint(e3, ecs_iter_get_var(&it, x_var));
-    {
-        Position *p = ecs_field(&it, Position, 1);
-        test_assert(p == NULL);
-    }
 
     test_bool(false, ecs_query_next(&it));
 
@@ -481,6 +465,9 @@ void Sparse_2_sparse(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t e1 = ecs_insert(world, 
         ecs_value(Position, {10, 20}),
         ecs_value(Velocity, {1,  2}));
@@ -493,43 +480,39 @@ void Sparse_2_sparse(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(3, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 0);
         test_assert(v != NULL);
         test_int(v->x, 1); test_int(v->y, 2);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 1);
         test_assert(v != NULL);
         test_int(v->x, 3); test_int(v->y, 4);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e3, it.entities[0]);
+    test_uint(e3, it.entities[2]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 2);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 2);
         test_assert(v != NULL);
         test_int(v->x, 5); test_int(v->y, 6);
     }
@@ -555,6 +538,9 @@ void Sparse_2_sparse_and_regular(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+    test_bool(false, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t e1 = ecs_insert(world, 
         ecs_value(Position, {10, 20}),
         ecs_value(Velocity, {1,  2}));
@@ -567,45 +553,41 @@ void Sparse_2_sparse_and_regular(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(3, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
     {
         Velocity *v = ecs_field(&it, Velocity, 1);
         test_assert(v != NULL);
-        test_int(v->x, 1); test_int(v->y, 2);
+        test_int(v[0].x, 1); test_int(v[0].y, 2);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
     {
         Velocity *v = ecs_field(&it, Velocity, 1);
         test_assert(v != NULL);
-        test_int(v->x, 3); test_int(v->y, 4);
+        test_int(v[1].x, 3); test_int(v[1].y, 4);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e3, it.entities[0]);
+    test_uint(e3, it.entities[2]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 2);
         test_assert(p != NULL);
         test_int(p->x, 50); test_int(p->y, 60);
     }
     {
         Velocity *v = ecs_field(&it, Velocity, 1);
         test_assert(v != NULL);
-        test_int(v->x, 5); test_int(v->y, 6);
+        test_int(v[2].x, 5); test_int(v[2].y, 6);
     }
 
     test_bool(false, ecs_query_next(&it));
@@ -629,6 +611,9 @@ void Sparse_2_regular_and_sparse(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t e1 = ecs_insert(world, 
         ecs_value(Position, {10, 20}),
         ecs_value(Velocity, {1,  2}));
@@ -641,43 +626,39 @@ void Sparse_2_regular_and_sparse(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(3, it.count);
     test_uint(e1, it.entities[0]);
     {
         Position *p = ecs_field(&it, Position, 0);
         test_assert(p != NULL);
-        test_int(p->x, 10); test_int(p->y, 20);
+        test_int(p[0].x, 10); test_int(p[0].y, 20);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 0);
         test_assert(v != NULL);
         test_int(v->x, 1); test_int(v->y, 2);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
         Position *p = ecs_field(&it, Position, 0);
         test_assert(p != NULL);
-        test_int(p->x, 30); test_int(p->y, 40);
+        test_int(p[1].x, 30); test_int(p[1].y, 40);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 1);
         test_assert(v != NULL);
         test_int(v->x, 3); test_int(v->y, 4);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e3, it.entities[0]);
+    test_uint(e3, it.entities[2]);
     {
         Position *p = ecs_field(&it, Position, 0);
         test_assert(p != NULL);
-        test_int(p->x, 50); test_int(p->y, 60);
+        test_int(p[2].x, 50); test_int(p[2].y, 60);
     }
     {
-        Velocity *v = ecs_field(&it, Velocity, 1);
+        Velocity *v = ecs_field_at(&it, Velocity, 1, 2);
         test_assert(v != NULL);
         test_int(v->x, 5); test_int(v->y, 6);
     }
@@ -703,6 +684,8 @@ void Sparse_1_sparse_self(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {30, 40}));
@@ -711,19 +694,17 @@ void Sparse_1_sparse_self(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
@@ -749,6 +730,8 @@ void Sparse_1_sparse_up(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {30, 40}));
@@ -761,7 +744,7 @@ void Sparse_1_sparse_up(void) {
     test_uint(e1, it.entities[0]);
     test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
@@ -771,7 +754,7 @@ void Sparse_1_sparse_up(void) {
     test_uint(e3, it.entities[0]);
     test_uint(e4, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
@@ -797,6 +780,8 @@ void Sparse_1_sparse_self_up(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(true, !!(q->row_fields & (1llu << 0)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, ecs_value(Position, {30, 40}));
@@ -805,37 +790,33 @@ void Sparse_1_sparse_self_up(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e3, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e4, it.entities[0]);
+    test_uint(e4, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
 
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 0);
+        Position *p = ecs_field_at(&it, Position, 0, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
@@ -862,6 +843,9 @@ void Sparse_1_sparse_written_self(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {30, 40}));
@@ -870,19 +854,17 @@ void Sparse_1_sparse_written_self(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
@@ -909,6 +891,9 @@ void Sparse_1_sparse_written_up(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {30, 40}));
@@ -921,7 +906,7 @@ void Sparse_1_sparse_written_up(void) {
     test_uint(e1, it.entities[0]);
     test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
@@ -931,7 +916,7 @@ void Sparse_1_sparse_written_up(void) {
     test_uint(e3, it.entities[0]);
     test_uint(e4, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
@@ -958,6 +943,9 @@ void Sparse_1_sparse_written_self_up(void) {
     });
     test_assert(q != NULL);
 
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+    test_bool(true, !!(q->row_fields & (1llu << 1)));
+
     ecs_entity_t base = ecs_insert(world, {EcsPrefab}, ecs_value(Position, {1, 2}));
     ecs_entity_t e1 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {10, 20}));
     ecs_entity_t e2 = ecs_insert(world, {ecs_isa(base)}, {Foo}, ecs_value(Position, {30, 40}));
@@ -966,37 +954,33 @@ void Sparse_1_sparse_written_self_up(void) {
 
     ecs_iter_t it = ecs_query_iter(world, q);
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e1, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 10); test_int(p->y, 20);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e2, it.entities[0]);
+    test_uint(e2, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 1);
         test_assert(p != NULL);
         test_int(p->x, 30); test_int(p->y, 40);
     }
 
     test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
+    test_int(2, it.count);
     test_uint(e3, it.entities[0]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 0);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
 
-    test_bool(true, ecs_query_next(&it));
-    test_int(1, it.count);
-    test_uint(e4, it.entities[0]);
+    test_uint(e4, it.entities[1]);
     {
-        Position *p = ecs_field(&it, Position, 1);
+        Position *p = ecs_field_at(&it, Position, 1, 1);
         test_assert(p != NULL);
         test_int(p->x, 1); test_int(p->y, 2);
     }
@@ -1007,3 +991,65 @@ void Sparse_1_sparse_written_self_up(void) {
 
     ecs_fini(world);
 }
+
+void Sparse_sparse_0_src_only_term(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0)",
+        .cache_kind = cache_kind
+    });
+
+    test_assert(q != NULL);
+
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+
+    ecs_iter_t it = ecs_query_iter(world, q);
+    test_bool(true, ecs_query_next(&it));
+    test_int(0, it.count);
+    test_uint(ecs_id(Position), ecs_field_id(&it, 0));
+    test_bool(false, ecs_field_is_set(&it, 0));
+    test_bool(false, ecs_query_next(&it));
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Sparse_sparse_0_src(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Foo);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_entity_t e = ecs_new_w(world, Foo);
+
+    ecs_query_t *q = ecs_query(world, {
+        .expr = "Position(#0), Foo",
+        .cache_kind = cache_kind
+    });
+
+    test_assert(q != NULL);
+
+    test_bool(false, !!(q->row_fields & (1llu << 0)));
+
+    ecs_iter_t it = ecs_query_iter(world, q);
+    test_bool(true, ecs_query_next(&it));
+    test_int(1, it.count);
+    test_uint(e, it.entities[0]);
+    test_uint(ecs_id(Position), ecs_field_id(&it, 0));
+    test_uint(Foo, ecs_field_id(&it, 1));
+    test_bool(false, ecs_field_is_set(&it, 0));
+    test_bool(true, ecs_field_is_set(&it, 1));
+    test_bool(false, ecs_query_next(&it));
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
