@@ -1,5 +1,82 @@
 #include <query.h>
 
+static
+void query_flags_to_str(uint64_t value) {
+    if (value & EcsQueryMatchPrefab) {
+        printf("EcsQueryMatchPrefab|");
+    }
+    if (value & EcsQueryMatchDisabled) {
+        printf("EcsQueryMatchDisabled|");
+    }
+    if (value & EcsQueryMatchEmptyTables) {
+        printf("EcsQueryMatchEmptyTables|");
+    }
+    if (value & EcsQueryAllowUnresolvedByName) {
+        printf("EcsQueryAllowUnresolvedByName|");
+    }
+    if (value & EcsQueryTableOnly) {
+        printf("EcsQueryTableOnly|");
+    }
+    if (value & EcsQueryMatchThis) {
+        printf("EcsQueryMatchThis|");
+    }
+    if (value & EcsQueryMatchOnlyThis) {
+        printf("EcsQueryMatchOnlyThis|");
+    }
+    if (value & EcsQueryMatchOnlySelf) {
+        printf("EcsQueryMatchOnlySelf|");
+    }
+    if (value & EcsQueryMatchWildcards) {
+        printf("EcsQueryMatchWildcards|");
+    }
+    if (value & EcsQueryHasCondSet) {
+        printf("EcsQueryHasCondSet|");
+    }
+    if (value & EcsQueryHasPred) {
+        printf("EcsQueryHasPred|");
+    }
+    if (value & EcsQueryHasScopes) {
+        printf("EcsQueryHasScopes|");
+    }
+    if (value & EcsQueryHasRefs) {
+        printf("EcsQueryHasRefs|");
+    }
+    if (value & EcsQueryHasOutTerms) {
+        printf("EcsQueryHasOutTerms|");
+    }
+    if (value & EcsQueryHasNonThisOutTerms) {
+        printf("EcsQueryHasNonThisOutTerms|");
+    }
+    if (value & EcsQueryHasMonitor) {
+        printf("EcsQueryHasMonitor|");
+    }
+    if (value & EcsQueryIsTrivial) {
+        printf("EcsQueryIsTrivial|");
+    }
+    if (value & EcsQueryHasCacheable) {
+        printf("EcsQueryHasCacheable|");
+    }
+    if (value & EcsQueryIsCacheable) {
+        printf("EcsQueryIsCacheable|");
+    }
+    if (value & EcsQueryHasTableThisVar) {
+        printf("EcsQueryHasTableThisVar|");
+    }
+    if (value & EcsQueryCacheYieldEmptyTables) {
+        printf("EcsQueryCacheYieldEmptyTables|");
+    }
+    printf("\n");
+}
+
+#define test_query_flags(expect, value)\
+    if ((expect) != (value)) {\
+        printf("expected: ");\
+        query_flags_to_str(expect);\
+        printf("got:      ");\
+        query_flags_to_str(value);\
+        test_uint(expect, value);\
+    }
+
 void Validator_validate_1_term(void) {
     ecs_world_t *world = ecs_mini();
 
@@ -1732,15 +1809,13 @@ void Validator_validate_filter(void) {
 
     ecs_query_t *q = ecs_query(world, {
         .terms = {
-            {TagA}
-        },
-        .flags = EcsQueryNoData
+            {TagA, .inout = EcsInOutNone}
+        }
     });
 
     test_int(q->term_count, 1);
     test_uint(q->terms[0].id, TagA);
     test_int(q->terms[0].oper, EcsAnd);
-    test_int(q->terms[0].inout, EcsInOutNone);
 
     ecs_query_fini(q);
 
@@ -1861,8 +1936,6 @@ void Validator_validate_w_tag_term_is_no_data(void) {
         .terms = {{ Foo }}
     });
 
-    test_assert(q->flags & EcsQueryNoData);
-
     ecs_iter_t it = ecs_query_iter(world, q);
     test_assert(it.flags & EcsIterNoData);
     ecs_iter_fini(&it);
@@ -1881,8 +1954,6 @@ void Validator_validate_w_inout_none_term_is_no_data(void) {
         .terms = {{ ecs_id(Position), .inout = EcsInOutNone }}
     });
 
-    test_assert(q->flags & EcsQueryNoData);
-
     ecs_iter_t it = ecs_query_iter(world, q);
     test_assert(it.flags & EcsIterNoData);
     ecs_iter_fini(&it);
@@ -1905,8 +1976,6 @@ void Validator_validate_w_tag_and_inout_none_term_is_no_data(void) {
         }
     });
 
-    test_assert(q->flags & EcsQueryNoData);
-
     ecs_iter_t it = ecs_query_iter(world, q);
     test_assert(it.flags & EcsIterNoData);
     ecs_iter_fini(&it);
@@ -1928,8 +1997,6 @@ void Validator_validate_w_not_term_is_no_data(void) {
         }
     });
 
-    test_assert(q->flags & EcsQueryNoData);
-
     ecs_iter_t it = ecs_query_iter(world, q);
     test_assert(it.flags & EcsIterNoData);
     ecs_iter_fini(&it);
@@ -2934,3 +3001,596 @@ void Validator_validate_custom_trav_w_inherit_id_w_up(void) {
 
     ecs_fini(world);
 }
+
+void Validator_validate_simple_1_term_is_cacheable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_1_term_tag_is_cacheable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Tag);
+
+    ecs_new_w(world, Tag);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { Tag },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, Tag);
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, Tag|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_1_term_pair_is_cacheable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ecs_entity_t rel = ecs_new(world);
+    ecs_entity_t tgt = ecs_new(world);
+
+    ecs_new_w_pair(world, rel, tgt);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_pair(rel, tgt) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_pair(rel, tgt));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, rel|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].second.id, tgt|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_1_term_pair_recycled_is_cacheable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ecs_entity_t rel = ecs_new(world);
+    ecs_delete(world, rel);
+    rel = ecs_new(world);
+    test_assert((uint32_t)rel != rel);
+
+    ecs_entity_t tgt = ecs_new(world);
+    ecs_delete(world, tgt);
+    tgt = ecs_new(world);
+    test_assert((uint32_t)tgt != tgt);
+
+    ecs_new_w_pair(world, rel, tgt);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_pair(rel, tgt) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_pair(rel, tgt));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, rel|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].second.id, tgt|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_2_term_is_cacheable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+            { ecs_id(Velocity) },
+        }
+    });
+
+    test_int(q->term_count, 2);
+    test_int(q->field_count, 2);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_uint(q->terms[1].id, ecs_id(Velocity));
+    test_int(q->terms[1].oper, EcsAnd);
+    test_int(q->terms[1].field_index, 1);
+    test_uint(q->terms[1].first.id, ecs_id(Velocity)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[1].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[1].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert((q->data_fields & (1 << 0)));
+    test_assert((q->data_fields & (1 << 1)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_can_inherit(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_pair(world, ecs_id(Position), EcsOnInstantiate, EcsInherit);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsUp|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_can_toggle(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsCanToggle);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermIsToggle|EcsTermIsCacheable);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|EcsQueryHasCacheable|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_sparse(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermIsSparse);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_union(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Rel);
+
+    ecs_add_id(world, Rel, EcsUnion);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { Rel },
+        },
+        .cache_kind = EcsQueryCacheAuto
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, Rel);
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, Rel|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermIsCacheable|EcsTermIsTrivial);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_union_pair(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Rel);
+    ECS_TAG(world, Tgt);
+
+    ecs_add_id(world, Rel, EcsUnion);
+
+    ecs_new_w_pair(world, Rel, Tgt);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_pair(Rel, Tgt) },
+        },
+        .cache_kind = EcsQueryCacheAuto
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_pair(Rel, Tgt));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, Rel|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsUnion);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_transitive(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsTransitive);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_transitive_pair(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Tgt);
+
+    ecs_add_id(world, ecs_id(Position), EcsTransitive);
+
+    ecs_new_w_pair(world, ecs_id(Position), Tgt);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_pair_t(Position, Tgt) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_pair_t(Position, Tgt));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermTransitive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_reflexive(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_add_id(world, ecs_id(Position), EcsReflexive);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_id(Position) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_id(Position));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermIsCacheable|EcsTermIsTrivial|EcsTermKeepAlive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryIsTrivial|EcsQueryIsCacheable|
+        EcsQueryHasCacheable|EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_reflexive_pair(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Tgt);
+
+    ecs_add_id(world, ecs_id(Position), EcsReflexive);
+
+    ecs_new_w_pair(world, ecs_id(Position), Tgt);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { ecs_pair_t(Position, Tgt) },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, ecs_pair_t(Position, Tgt));
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, ecs_id(Position)|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermReflexive);
+
+    test_assert((q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar|EcsQueryHasOutTerms,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void Validator_validate_simple_w_inherited_component(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_TAG(world, Unit);
+    ECS_ENTITY(world, MeleeUnit, (IsA, Unit));
+
+    ecs_new_w(world, Unit);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {
+            { Unit },
+        }
+    });
+
+    test_int(q->term_count, 1);
+    test_int(q->field_count, 1);
+    test_int(q->var_count, 1);
+    test_assert(q->vars != NULL);
+    test_assert(q->vars[0] == NULL);
+
+    test_uint(q->terms[0].id, Unit);
+    test_int(q->terms[0].oper, EcsAnd);
+    test_int(q->terms[0].field_index, 0);
+    test_uint(q->terms[0].first.id, Unit|EcsSelf|EcsIsEntity);
+    test_uint(q->terms[0].src.id, EcsThis|EcsSelf|EcsIsVariable);
+    test_uint(q->terms[0].flags_, EcsTermKeepAlive|EcsTermIdInherited);
+
+    test_assert(!(q->data_fields & (1 << 0)));
+
+    test_query_flags(EcsQueryMatchOnlyThis|EcsQueryMatchThis|
+        EcsQueryMatchOnlySelf|EcsQueryHasTableThisVar,
+        q->flags);
+    
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
